@@ -1,11 +1,11 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
- * Version: NPL 1.1/GPL 2.0/LGPL 2.1
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public License
- * Version 1.1 (the "License"); you may not use this file except in
- * compliance with the License. You may obtain a copy of the License at
- * http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
  *
  * Software distributed under the License is distributed on an "AS IS" basis,
  * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
@@ -14,26 +14,27 @@
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is 
+ * The Initial Developer of the Original Code is
  * Netscape Communications Corporation.
  * Portions created by the Initial Developer are Copyright (C) 2001-2002
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
- *  Seth Spitzer <sspitzer@netscape.com>
- *  Dan Mosedale <dmose@netscape.com>
+ *   Seth Spitzer <sspitzer@netscape.com>
+ *   Dan Mosedale <dmose@netscape.com>
+ *   David Bienvenu <bienvenu@mozilla.org>
  *
  * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or 
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * either of the GNU General Public License Version 2 or later (the "GPL"),
+ * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
  * in which case the provisions of the GPL or the LGPL are applicable instead
  * of those above. If you wish to allow use of your version of this file only
  * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the NPL, indicate your
+ * use your version of this file under the terms of the MPL, indicate your
  * decision by deleting the provisions above and replace them with the notice
  * and other provisions required by the GPL or the LGPL. If you do not delete
  * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the NPL, the GPL or the LGPL.
+ * the terms of any one of the MPL, the GPL or the LGPL.
  *
  * ***** END LICENSE BLOCK ***** */
 
@@ -52,6 +53,13 @@
 #include "nsIImapIncomingServer.h"
 #include "nsIRDFService.h"
 #include "nsIRDFResource.h"
+#include "nsAppDirectoryServiceDefs.h"
+#include "nsIPrefService.h"
+#include "nsIPrefBranch.h"
+#include "nsIStringBundle.h"
+#include "nsDateTimeFormatCID.h"
+
+static NS_DEFINE_CID(kDateTimeFormatCID,    NS_DATETIMEFORMAT_CID);
 
 nsSpamSettings::nsSpamSettings()
 {
@@ -60,10 +68,15 @@ nsSpamSettings::nsSpamSettings()
   mMoveTargetMode = nsISpamSettings::MOVE_TARGET_MODE_ACCOUNT;
   mPurge = PR_FALSE;
   mPurgeInterval = 14; // 14 days
+
+  mServerFilterTrustFlags = 0;
+
   mUseWhiteList = PR_FALSE;
-  mLoggingEnabled = PR_FALSE;
-  mManualMark = PR_FALSE;
-  mManualMarkMode = nsISpamSettings::MANUAL_MARK_MODE_MOVE;
+  mUseServerFilter = PR_FALSE;
+
+  nsresult rv = NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR, getter_AddRefs(mLogFile));
+  if (NS_SUCCEEDED(rv))
+    mLogFile->Append(NS_LITERAL_STRING("junklog.html"));
 }
 
 nsSpamSettings::~nsSpamSettings()
@@ -102,26 +115,46 @@ NS_IMETHODIMP nsSpamSettings::SetMoveTargetMode(PRInt32 aMoveTargetMode)
   return NS_OK;
 }
 
-NS_IMETHODIMP 
-nsSpamSettings::GetManualMarkMode(PRInt32 *aManualMarkMode)
+NS_IMETHODIMP nsSpamSettings::GetManualMark(PRBool *aManualMark)
+{
+  NS_ENSURE_ARG_POINTER(aManualMark);
+  nsresult rv;
+  nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
+  NS_ENSURE_SUCCESS(rv, rv);
+  return prefBranch->GetBoolPref("mail.spam.manualMark", aManualMark);
+}
+
+NS_IMETHODIMP nsSpamSettings::GetManualMarkMode(PRInt32 *aManualMarkMode)
 {
   NS_ENSURE_ARG_POINTER(aManualMarkMode);
-  *aManualMarkMode = mManualMarkMode;
-  return NS_OK;
+  nsresult rv;
+  nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
+  NS_ENSURE_SUCCESS(rv, rv);
+  return prefBranch->GetIntPref("mail.spam.manualMarkMode", aManualMarkMode);
 }
 
-NS_IMETHODIMP nsSpamSettings::SetManualMarkMode(PRInt32 aManualMarkMode)
+NS_IMETHODIMP nsSpamSettings::GetLoggingEnabled(PRBool *aLoggingEnabled)
 { 
-  NS_ASSERTION((aManualMarkMode == nsISpamSettings::MANUAL_MARK_MODE_MOVE || aManualMarkMode == nsISpamSettings::MANUAL_MARK_MODE_DELETE), "bad manual mark mode");
-  mManualMarkMode = aManualMarkMode;
-  return NS_OK;
+  NS_ENSURE_ARG_POINTER(aLoggingEnabled);
+  nsresult rv;
+  nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
+  NS_ENSURE_SUCCESS(rv, rv);
+  return prefBranch->GetBoolPref("mail.spam.logging.enabled", aLoggingEnabled);
+}
+
+NS_IMETHODIMP nsSpamSettings::GetMarkAsReadOnSpam(PRBool *aMarkAsReadOnSpam)
+{
+  NS_ENSURE_ARG_POINTER(aMarkAsReadOnSpam);
+  nsresult rv;
+  nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
+  NS_ENSURE_SUCCESS(rv, rv);
+  return prefBranch->GetBoolPref("mail.spam.markAsReadOnSpam", aMarkAsReadOnSpam);
 }
 
-NS_IMPL_GETSET(nsSpamSettings, LoggingEnabled, PRBool, mLoggingEnabled)
 NS_IMPL_GETSET(nsSpamSettings, MoveOnSpam, PRBool, mMoveOnSpam)
 NS_IMPL_GETSET(nsSpamSettings, Purge, PRBool, mPurge)
 NS_IMPL_GETSET(nsSpamSettings, UseWhiteList, PRBool, mUseWhiteList)
-NS_IMPL_GETSET(nsSpamSettings, ManualMark, PRBool, mManualMark)
+NS_IMPL_GETSET(nsSpamSettings, UseServerFilter, PRBool, mUseServerFilter)
 
 NS_IMETHODIMP nsSpamSettings::GetWhiteListAbURI(char * *aWhiteListAbURI)
 {
@@ -189,6 +222,9 @@ nsSpamSettings::SetLogStream(nsIOutputStream *aLogStream)
   return NS_OK;
 }
 
+ #define LOG_HEADER "<head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"></head>"
+ #define LOG_HEADER_LEN (strlen(LOG_HEADER))
+ 
 NS_IMETHODIMP
 nsSpamSettings::GetLogStream(nsIOutputStream **aLogStream)
 {
@@ -197,18 +233,7 @@ nsSpamSettings::GetLogStream(nsIOutputStream **aLogStream)
   nsresult rv;
 
   if (!mLogStream) {
-    nsCOMPtr <nsIFileSpec> file;
-    rv = GetLogFileSpec(getter_AddRefs(file));
-    NS_ENSURE_SUCCESS(rv,rv);
-
-    nsXPIDLCString nativePath;
-    rv = file->GetNativePath(getter_Copies(nativePath));
-    NS_ENSURE_SUCCESS(rv,rv);
-
-    nsCOMPtr <nsILocalFile> logFile = do_CreateInstance(NS_LOCAL_FILE_CONTRACTID, &rv);
-    NS_ENSURE_SUCCESS(rv,rv);
-
-    rv = logFile->InitWithNativePath(nsDependentCString(nativePath));
+    nsCOMPtr <nsILocalFile> logFile = do_QueryInterface(mLogFile, &rv);
     NS_ENSURE_SUCCESS(rv,rv);
 
     // append to the end of the log file
@@ -217,123 +242,125 @@ nsSpamSettings::GetLogStream(nsIOutputStream **aLogStream)
                                    PR_CREATE_FILE | PR_WRONLY | PR_APPEND,
                                    0600);
     NS_ENSURE_SUCCESS(rv,rv);
-
-    if (!mLogStream)
-      return NS_ERROR_FAILURE;
   }
  
   NS_ADDREF(*aLogStream = mLogStream);
   return NS_OK;
 }
 
-NS_IMETHODIMP nsSpamSettings::GetServer(nsIMsgIncomingServer **aServer)
+NS_IMETHODIMP nsSpamSettings::Initialize(nsIMsgIncomingServer *aServer)
 {
   NS_ENSURE_ARG_POINTER(aServer);
-  NS_IF_ADDREF(*aServer = mServer);
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsSpamSettings::SetServer(nsIMsgIncomingServer *aServer)
-{
-  mServer = aServer;
-  return NS_OK;
-}
-
-nsresult
-nsSpamSettings::GetLogFileSpec(nsIFileSpec **aFileSpec)
-{
-  NS_ENSURE_ARG_POINTER(aFileSpec);
-
-  nsCOMPtr <nsIMsgIncomingServer> server;
-  nsresult rv = GetServer(getter_AddRefs(server));
-  NS_ENSURE_SUCCESS(rv,rv);
-
-  NS_ASSERTION(server, "are you trying to use a cloned spam settings?");
-  if (!server)
-    return NS_ERROR_FAILURE;
-
-  rv = server->GetLocalPath(aFileSpec);
-  NS_ENSURE_SUCCESS(rv,rv);
-
-  rv = (*aFileSpec)->AppendRelativeUnixPath("junklog.html");
-  NS_ENSURE_SUCCESS(rv,rv);
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsSpamSettings::GetLogURL(char * *aLogURL)
-{
-  NS_ENSURE_ARG_POINTER(aLogURL);
-
-  nsCOMPtr <nsIFileSpec> file;
-  nsresult rv = GetLogFileSpec(getter_AddRefs(file));
-  NS_ENSURE_SUCCESS(rv,rv);
-  
-  rv = file->GetURLString(aLogURL);
-  NS_ENSURE_SUCCESS(rv,rv);
-  return NS_OK;
-}
-
-nsresult nsSpamSettings::TruncateLog()
-{
-  // this will flush and close the steam
-  nsresult rv = SetLogStream(nsnull);
-  NS_ENSURE_SUCCESS(rv,rv);
-
-  nsCOMPtr <nsIFileSpec> file;
-  rv = GetLogFileSpec(getter_AddRefs(file));
-  NS_ENSURE_SUCCESS(rv,rv);
-
-  rv = file->Truncate(0);
-  NS_ENSURE_SUCCESS(rv,rv);
-  return rv;
-}
-
-NS_IMETHODIMP nsSpamSettings::ClearLog()
-{
-  PRBool loggingEnabled = mLoggingEnabled;
-  
-  // disable logging while clearing
-  mLoggingEnabled = PR_FALSE;
-
   nsresult rv;
-  rv = TruncateLog();
-  NS_ASSERTION(NS_SUCCEEDED(rv), "failed to truncate filter log");
+  PRInt32 spamLevel;
+  rv = aServer->GetIntValue("spamLevel", &spamLevel);
+  NS_ENSURE_SUCCESS(rv, rv);
+  rv = SetLevel(spamLevel);
+  NS_ENSURE_SUCCESS(rv, rv);
 
-  mLoggingEnabled = loggingEnabled;
-  return NS_OK;
-}
+  PRBool moveOnSpam;
+  rv = aServer->GetBoolValue("moveOnSpam", &moveOnSpam);
+  NS_ENSURE_SUCCESS(rv, rv);
+  rv = SetMoveOnSpam(moveOnSpam);
+  NS_ENSURE_SUCCESS(rv, rv);
 
-NS_IMETHODIMP nsSpamSettings::EnsureLogFile()
-{
-  nsCOMPtr <nsIFileSpec> file;
-  nsresult rv = GetLogFileSpec(getter_AddRefs(file));
+  PRInt32 moveTargetMode;
+  rv = aServer->GetIntValue("moveTargetMode", &moveTargetMode);
+  NS_ENSURE_SUCCESS(rv, rv);
+  rv = SetMoveTargetMode(moveTargetMode);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsXPIDLCString spamActionTargetAccount;
+  rv = aServer->GetCharValue("spamActionTargetAccount", getter_Copies(spamActionTargetAccount));
+  NS_ENSURE_SUCCESS(rv, rv);
+  rv = SetActionTargetAccount(spamActionTargetAccount);    
   NS_ENSURE_SUCCESS(rv,rv);
 
-  PRBool exists;
-  rv = file->Exists(&exists);
-  if (NS_SUCCEEDED(rv) && !exists) {
-    rv = file->Touch();
-    NS_ENSURE_SUCCESS(rv,rv);
-  }
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsSpamSettings::FlushLogIfNecessary()
-{
-  // only flush the log if we are logging
-  PRBool loggingEnabled = PR_FALSE;
-  nsresult rv = GetLoggingEnabled(&loggingEnabled);
+  nsXPIDLCString spamActionTargetFolder;
+  rv = aServer->GetCharValue("spamActionTargetFolder", getter_Copies(spamActionTargetFolder));
+  NS_ENSURE_SUCCESS(rv, rv);
+  rv = SetActionTargetFolder(spamActionTargetFolder);
   NS_ENSURE_SUCCESS(rv,rv);
 
-  if (loggingEnabled) 
+  PRBool useWhiteList;
+  rv = aServer->GetBoolValue("useWhiteList", &useWhiteList);
+  NS_ENSURE_SUCCESS(rv, rv);
+  rv = SetUseWhiteList(useWhiteList);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsXPIDLCString whiteListAbURI;
+  rv = aServer->GetCharValue("whiteListAbURI", getter_Copies(whiteListAbURI));
+  NS_ENSURE_SUCCESS(rv, rv);
+  rv = SetWhiteListAbURI(whiteListAbURI);
+  NS_ENSURE_SUCCESS(rv,rv);
+  
+  PRBool purgeSpam;
+  rv = aServer->GetBoolValue("purgeSpam", &purgeSpam);
+  NS_ENSURE_SUCCESS(rv, rv);
+  rv = SetPurge(purgeSpam);
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  PRInt32 purgeSpamInterval;
+  rv = aServer->GetIntValue("purgeSpamInterval", &purgeSpamInterval);
+  NS_ENSURE_SUCCESS(rv, rv);
+  rv = SetPurgeInterval(purgeSpamInterval);
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  PRBool useServerFilter;
+  rv = aServer->GetBoolValue("useServerFilter", &useServerFilter);
+  NS_ENSURE_SUCCESS(rv, rv);
+  rv = SetUseServerFilter(useServerFilter);
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  nsXPIDLCString serverFilterName;
+  rv = aServer->GetCharValue("serverFilterName", getter_Copies(serverFilterName));
+  if (NS_SUCCEEDED(rv))
+    SetServerFilterName(serverFilterName);
+  PRInt32 serverFilterTrustFlags = 0;
+  rv = aServer->GetIntValue("serverFilterTrustFlags", &serverFilterTrustFlags);
+  NS_ENSURE_SUCCESS(rv,rv);
+  rv = SetServerFilterTrustFlags(serverFilterTrustFlags);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  return UpdateJunkFolderState();
+}
+
+nsresult nsSpamSettings::UpdateJunkFolderState()
+{
+  nsresult rv;
+
+  // if the spam folder uri changed on us, we need to unset the junk flag
+  // on the old spam folder
+  nsXPIDLCString newJunkFolderURI;
+  rv = GetSpamFolderURI(getter_Copies(newJunkFolderURI));
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  if (!mCurrentJunkFolderURI.IsEmpty() && !mCurrentJunkFolderURI.Equals(newJunkFolderURI))
   {
-    nsCOMPtr <nsIOutputStream> logStream;
-    rv = GetLogStream(getter_AddRefs(logStream));    
-    if (NS_SUCCEEDED(rv) && logStream) {
-      rv = logStream->Flush();
-      NS_ENSURE_SUCCESS(rv,rv);
-    }
+    nsCOMPtr<nsIMsgFolder> oldJunkFolder;
+    rv = GetExistingFolder(mCurrentJunkFolderURI.get(), getter_AddRefs(oldJunkFolder));
+    if (NS_SUCCEEDED(rv) && oldJunkFolder) 
+    {
+      // remove the MSG_FOLDER_FLAG_JUNK on the old junk folder
+      // XXX TODO
+      // JUNK MAIL RELATED
+      // (in ClearFlag?) we need to make sure that this folder
+      // is not a the junk folder for another account
+      // the same goes for set flag.  have fun with all that.
+      oldJunkFolder->ClearFlag(MSG_FOLDER_FLAG_JUNK);
   }
+}
+
+  mCurrentJunkFolderURI = newJunkFolderURI;
+
+  // only try to create the junk folder if we are moving junk
+  // and we have a non-empty uri
+  if (mMoveOnSpam && !mCurrentJunkFolderURI.IsEmpty()) {
+    // as the url listener, the spam settings will set the MSG_FOLDER_FLAG_JUNK folder flag
+    // on the junk mail folder, after it is created
+    rv = GetOrCreateFolder(mCurrentJunkFolderURI, this);
+  }
+
   return rv;
 }
 
@@ -345,9 +372,8 @@ NS_IMETHODIMP nsSpamSettings::Clone(nsISpamSettings *aSpamSettings)
   NS_ENSURE_SUCCESS(rv,rv);
 
   (void)aSpamSettings->GetMoveOnSpam(&mMoveOnSpam);
-  (void)aSpamSettings->GetManualMark(&mManualMark); 
-  (void)aSpamSettings->GetManualMarkMode(&mManualMarkMode); 
   (void)aSpamSettings->GetPurge(&mPurge); 
+  (void)aSpamSettings->GetUseServerFilter(&mUseServerFilter);
 
   rv = aSpamSettings->GetPurgeInterval(&mPurgeInterval); 
   NS_ENSURE_SUCCESS(rv,rv);
@@ -373,10 +399,9 @@ NS_IMETHODIMP nsSpamSettings::Clone(nsISpamSettings *aSpamSettings)
   NS_ENSURE_SUCCESS(rv,rv);
   mWhiteListAbURI = whiteListAbURI;
   
-  PRBool loggingEnabled;
-  rv = aSpamSettings->GetLoggingEnabled(&loggingEnabled);
-  NS_ENSURE_SUCCESS(rv,rv);
-  mLoggingEnabled = loggingEnabled;
+  aSpamSettings->GetServerFilterName(mServerFilterName);
+  aSpamSettings->GetServerFilterTrustFlags(&mServerFilterTrustFlags);
+
   return rv;
 }
 
@@ -438,6 +463,19 @@ NS_IMETHODIMP nsSpamSettings::GetSpamFolderURI(char **aSpamFolderURI)
     return rv;
 }
 
+NS_IMETHODIMP nsSpamSettings::GetServerFilterName(nsACString &aFilterName)
+{
+  aFilterName = mServerFilterName;
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsSpamSettings::SetServerFilterName(const nsACString &aFilterName)
+{
+  mServerFilterName = aFilterName;
+  return NS_OK;
+}
+
+NS_IMPL_GETSET(nsSpamSettings, ServerFilterTrustFlags, PRBool, mServerFilterTrustFlags)
 
 #define LOG_ENTRY_START_TAG "<p>\n"
 #define LOG_ENTRY_START_TAG_LEN (strlen(LOG_ENTRY_START_TAG))
@@ -454,31 +492,54 @@ NS_IMETHODIMP nsSpamSettings::LogJunkHit(nsIMsgDBHdr *aMsgHdr, PRBool aMoveMessa
     return NS_OK;
 
   PRTime date;
-  char dateStr[40];	/* 30 probably not enough */
   
-  nsXPIDLCString author;
-  nsXPIDLCString subject;
+  nsXPIDLString authorValue;
+  nsXPIDLString subjectValue;
+  nsXPIDLString dateValue;
   
-  rv = aMsgHdr->GetDate(&date);
+  (void)aMsgHdr->GetDate(&date);
   PRExplodedTime exploded;
   PR_ExplodeTime(date, PR_LocalTimeParameters, &exploded);
-  // XXX Temporary until logging is fully localized
-  PR_FormatTimeUSEnglish(dateStr, sizeof(dateStr), "%Y-%m-%d %H:%M:%S", &exploded);
+
+  if (!mDateFormatter)
+  {
+    mDateFormatter = do_CreateInstance(kDateTimeFormatCID, &rv);
+    NS_ENSURE_SUCCESS(rv, rv);
+    if (!mDateFormatter)
+    {
+      return NS_ERROR_FAILURE;
+    }
+  }
+  mDateFormatter->FormatPRExplodedTime(nsnull, kDateFormatShort,
+                                      kTimeFormatSeconds, &exploded, 
+                                      dateValue);
   
-  aMsgHdr->GetAuthor(getter_Copies(author));
-  aMsgHdr->GetSubject(getter_Copies(subject));
+  (void)aMsgHdr->GetMime2DecodedAuthor(getter_Copies(authorValue));
+  (void)aMsgHdr->GetMime2DecodedSubject(getter_Copies(subjectValue));
   
   nsCString buffer;
   // this is big enough to hold a log entry.  
   // do this so we avoid growing and copying as we append to the log.
   buffer.SetCapacity(512);  
   
-  buffer = "Detected junk message from ";
-  buffer +=  (const char*)author;
-  buffer +=  " - ";
-  buffer +=  (const char *)subject;
-  buffer +=  " at ";
-  buffer +=  dateStr;
+  nsCOMPtr<nsIStringBundleService> bundleService =
+    do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+  
+  nsCOMPtr<nsIStringBundle> bundle;
+  rv = bundleService->CreateBundle("chrome://messenger/locale/filter.properties",
+    getter_AddRefs(bundle));
+  NS_ENSURE_SUCCESS(rv, rv);
+  
+  const PRUnichar *junkLogDetectFormatStrings[3] = { authorValue.get(), subjectValue.get(), dateValue.get() };
+  nsXPIDLString junkLogDetectStr;
+  rv = bundle->FormatStringFromName(
+    NS_LITERAL_STRING("junkLogDetectStr").get(),
+    junkLogDetectFormatStrings, 3,
+    getter_Copies(junkLogDetectStr));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  buffer += NS_ConvertUTF16toUTF8(junkLogDetectStr);
   buffer +=  "\n";
   
   if (aMoveMessage) {
@@ -487,15 +548,23 @@ NS_IMETHODIMP nsSpamSettings::LogJunkHit(nsIMsgDBHdr *aMsgHdr, PRBool aMoveMessa
     
     nsXPIDLCString junkFolderURI;
     rv = GetSpamFolderURI(getter_Copies(junkFolderURI));
-    NS_ENSURE_SUCCESS(rv,rv);
+    NS_ENSURE_SUCCESS(rv, rv);
 
-    buffer += "Move message id = ";
-    buffer += msgId.get();
-    buffer += " to ";
-    buffer += junkFolderURI.get();
+    NS_ConvertASCIItoUTF16 msgIdValue(msgId);
+    NS_ConvertASCIItoUTF16 junkFolderURIValue(junkFolderURI);
+    
+    const PRUnichar *logMoveFormatStrings[2] = { msgIdValue.get(), junkFolderURIValue.get() };
+    nsXPIDLString logMoveStr;
+    rv = bundle->FormatStringFromName(
+      NS_LITERAL_STRING("logMoveStr").get(),
+      logMoveFormatStrings, 2,
+      getter_Copies(logMoveStr));
+    NS_ENSURE_SUCCESS(rv, rv);
+    
+    buffer += NS_ConvertUTF16toUTF8(logMoveStr);
     buffer += "\n";
   }
-  
+
   return LogJunkString(buffer.get());
 }
 
