@@ -76,12 +76,16 @@
 #include "nsIRssIncomingServer.h"
 #include "nsIMsgFolder.h"
 #include "nsIMsgMessageService.h"
+#include "nsIMsgAccountManager.h"
+
 
 static NS_DEFINE_CID(kImapUrlCID, NS_IMAPURL_CID);
 static NS_DEFINE_CID(kCMailboxUrl, NS_MAILBOXURL_CID);
 static NS_DEFINE_CID(kCNntpUrlCID, NS_NNTPURL_CID);
 
 #define ILLEGAL_FOLDER_CHARS ";#"
+#define ILLEGAL_FOLDER_CHARS_AS_FIRST_LETTER "." 
+#define ILLEGAL_FOLDER_CHARS_AS_LAST_LETTER  ".~"
 
 #define NS_PASSWORDMANAGER_CATEGORY "passwordmanager"
 static PRBool gInitPasswordManager = PR_FALSE;
@@ -171,76 +175,119 @@ nsresult CreateStartupUrl(const char *uri, nsIURI** aUrl)
 }
 
 
-// Where should this live? It's a utility used to convert a string priority, e.g., "High, Low, Normal" to an enum.
-// Perhaps we should have an interface that groups together all these utilities...
-nsresult NS_MsgGetPriorityFromString(const char *priority, nsMsgPriorityValue *outPriority)
+// Where should this live? It's a utility used to convert a string priority,
+//  e.g., "High, Low, Normal" to an enum.
+// Perhaps we should have an interface that groups together all these
+//  utilities...
+nsresult NS_MsgGetPriorityFromString(
+           const char * const priority,
+           nsMsgPriorityValue & outPriority)
 {
-	if (!outPriority)
-		return NS_ERROR_NULL_POINTER;
+  if (!priority)
+    return NS_ERROR_NULL_POINTER;
 
-	nsMsgPriorityValue retPriority = nsMsgPriority::normal;
+  // Note: Checking the values separately and _before_ the names,
+  //        hoping for a much faster match;
+  //       Only _drawback_, as "priority" handling is not truly specified:
+  //        some softwares may have the number meanings reversed (1=Lowest) !?
+  if (PL_strchr(priority, '1'))
+    outPriority = nsMsgPriority::highest;
+  else if (PL_strchr(priority, '2'))
+    outPriority = nsMsgPriority::high;
+  else if (PL_strchr(priority, '3'))
+    outPriority = nsMsgPriority::normal;
+  else if (PL_strchr(priority, '4'))
+    outPriority = nsMsgPriority::low;
+  else if (PL_strchr(priority, '5'))
+    outPriority = nsMsgPriority::lowest;
+  else if (PL_strcasestr(priority, "Highest"))
+    outPriority = nsMsgPriority::highest;
+       // Important: "High" must be tested after "Highest" !
+  else if (PL_strcasestr(priority, "High") ||
+           PL_strcasestr(priority, "Urgent"))
+    outPriority = nsMsgPriority::high;
+  else if (PL_strcasestr(priority, "Normal"))
+    outPriority = nsMsgPriority::normal;
+  else if (PL_strcasestr(priority, "Lowest"))
+    outPriority = nsMsgPriority::lowest;
+       // Important: "Low" must be tested after "Lowest" !
+  else if (PL_strcasestr(priority, "Low") ||
+           PL_strcasestr(priority, "Non-urgent"))
+    outPriority = nsMsgPriority::low;
+  else
+    // "Default" case gets default value.
+    outPriority = nsMsgPriority::Default;
 
-	if (PL_strcasestr(priority, "Normal") != NULL)
-		retPriority = nsMsgPriority::normal;
-	else if (PL_strcasestr(priority, "Lowest") != NULL)
-		retPriority = nsMsgPriority::lowest;
-	else if (PL_strcasestr(priority, "Highest") != NULL)
-		retPriority = nsMsgPriority::highest;
-	else if (PL_strcasestr(priority, "High") != NULL || 
-			 PL_strcasestr(priority, "Urgent") != NULL)
-		retPriority = nsMsgPriority::high;
-	else if (PL_strcasestr(priority, "Low") != NULL ||
-			 PL_strcasestr(priority, "Non-urgent") != NULL)
-		retPriority = nsMsgPriority::low;
-	else if (PL_strcasestr(priority, "1") != NULL)
-		retPriority = nsMsgPriority::highest;
-	else if (PL_strcasestr(priority, "2") != NULL)
-		retPriority = nsMsgPriority::high;
-	else if (PL_strcasestr(priority, "3") != NULL)
-		retPriority = nsMsgPriority::normal;
-	else if (PL_strcasestr(priority, "4") != NULL)
-		retPriority = nsMsgPriority::low;
-	else if (PL_strcasestr(priority, "5") != NULL)
-	    retPriority = nsMsgPriority::lowest;
-	else
-		retPriority = nsMsgPriority::normal;
-	*outPriority = retPriority;
-	return NS_OK;
-		//return nsMsgNoPriority;
+  return NS_OK;
 }
 
-
-nsresult NS_MsgGetUntranslatedPriorityName (nsMsgPriorityValue p, nsString *outName)
+nsresult NS_MsgGetPriorityValueString(
+           const nsMsgPriorityValue p,
+           nsACString & outValueString)
 {
-	if (!outName)
-		return NS_ERROR_NULL_POINTER;
-	switch (p)
-	{
-	case nsMsgPriority::notSet:
-	case nsMsgPriority::none:
-		outName->AssignLiteral("None");
-		break;
-	case nsMsgPriority::lowest:
-		outName->AssignLiteral("Lowest");
-		break;
-	case nsMsgPriority::low:
-		outName->AssignLiteral("Low");
-		break;
-	case nsMsgPriority::normal:
-		outName->AssignLiteral("Normal");
-		break;
-	case nsMsgPriority::high:
-		outName->AssignLiteral("High");
-		break;
-	case nsMsgPriority::highest:
-		outName->AssignLiteral("Highest");
-		break;
-	default:
-		NS_ASSERTION(PR_FALSE, "invalid priority value");
-	}
-	return NS_OK;
+  switch (p)
+  {
+    case nsMsgPriority::highest:
+      outValueString.AssignLiteral("1");
+      break;
+    case nsMsgPriority::high:
+      outValueString.AssignLiteral("2");
+      break;
+    case nsMsgPriority::normal:
+      outValueString.AssignLiteral("3");
+      break;
+    case nsMsgPriority::low:
+      outValueString.AssignLiteral("4");
+      break;
+    case nsMsgPriority::lowest:
+      outValueString.AssignLiteral("5");
+      break;
+    case nsMsgPriority::none:
+    case nsMsgPriority::notSet:
+      // Note: '0' is a "fake" value; we expect to never be in this case.
+      outValueString.AssignLiteral("0");
+      break;
+    default:
+      NS_ASSERTION(PR_FALSE, "invalid priority value");
+  }
+
+  return NS_OK;
 }
 
+nsresult NS_MsgGetUntranslatedPriorityName(
+           const nsMsgPriorityValue p,
+           nsACString & outName)
+{
+  switch (p)
+  {
+    case nsMsgPriority::highest:
+      outName.AssignLiteral("Highest");
+      break;
+    case nsMsgPriority::high:
+      outName.AssignLiteral("High");
+      break;
+    case nsMsgPriority::normal:
+      outName.AssignLiteral("Normal");
+      break;
+    case nsMsgPriority::low:
+      outName.AssignLiteral("Low");
+      break;
+    case nsMsgPriority::lowest:
+      outName.AssignLiteral("Lowest");
+      break;
+    case nsMsgPriority::none:
+    case nsMsgPriority::notSet:
+      // Note: 'None' is a "fake" value; we expect to never be in this case.
+      outName.AssignLiteral("None");
+      break;
+    default:
+      NS_ASSERTION(PR_FALSE, "invalid priority value");
+  }
+
+  return NS_OK;
+}
+
+
 /* this used to be XP_StringHash2 from xp_hash.c */
 /* phong's linear congruential hash  */
 static PRUint32 StringHash(const char *ubuf, PRInt32 len = -1)
@@ -294,6 +341,22 @@ nsresult NS_MsgHashIfNecessary(nsCAutoString &name)
   // certain filenames require hashing because they 
   // are too long or contain illegal characters
   PRInt32 illegalCharacterIndex = str.FindCharInSet(illegalChars);
+
+  // Need to check the first ('.') and last ('.' and '~') char
+  if (illegalCharacterIndex == kNotFound) 
+  {
+	NS_NAMED_LITERAL_CSTRING (illegalFirstChars, ILLEGAL_FOLDER_CHARS_AS_FIRST_LETTER);
+	NS_NAMED_LITERAL_CSTRING (illegalLastChars, ILLEGAL_FOLDER_CHARS_AS_LAST_LETTER);
+	  
+    PRInt32 lastIndex = str.Length() - 1;
+    if(str.FindCharInSet(illegalFirstChars) == 0)
+	  illegalCharacterIndex = 0;
+	else if(str.RFindCharInSet(illegalLastChars) == lastIndex)
+	  illegalCharacterIndex = lastIndex;
+	else
+	  illegalCharacterIndex = -1;
+  }
+
   char hashedname[MAX_LEN + 1];
   if (illegalCharacterIndex == kNotFound) 
   {
@@ -333,6 +396,21 @@ nsresult NS_MsgHashIfNecessary(nsAutoString &name)
   PRInt32 illegalCharacterIndex = name.FindCharInSet(
                                   FILE_PATH_SEPARATOR FILE_ILLEGAL_CHARACTERS ILLEGAL_FOLDER_CHARS);
 
+  // Need to check the first ('.') and last ('.' and '~') char
+  if (illegalCharacterIndex == kNotFound) 
+  {
+	NS_NAMED_LITERAL_STRING (illegalFirstChars, ILLEGAL_FOLDER_CHARS_AS_FIRST_LETTER);
+	NS_NAMED_LITERAL_STRING (illegalLastChars, ILLEGAL_FOLDER_CHARS_AS_LAST_LETTER);
+	  
+    PRInt32 lastIndex = name.Length() - 1;
+    if(name.FindCharInSet(illegalFirstChars) == 0)
+	  illegalCharacterIndex = 0;
+	else if(name.RFindCharInSet(illegalLastChars) == lastIndex)
+	  illegalCharacterIndex = lastIndex;
+	else
+	  illegalCharacterIndex = -1;
+  }
+
   char hashedname[9];
   PRInt32 keptLength = -1;
   if (illegalCharacterIndex != kNotFound) 
@@ -1231,3 +1309,131 @@ PRBool MsgHostDomainIsTrusted(nsCString &host, nsCString &trustedMailDomains)
   } while (*end);
   return domainIsTrusted;
 }
+
+nsresult MsgMailboxGetURI(const char *nativepath, nsCString &mailboxUri)
+{
+  
+  nsresult rv;
+  
+  nsCOMPtr<nsIMsgAccountManager> accountManager = 
+    do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
+  
+  NS_ENSURE_SUCCESS(rv, rv);
+  nsCOMPtr<nsISupportsArray> serverArray;
+  accountManager->GetAllServers(getter_AddRefs(serverArray));
+  
+  // do a char*->fileSpec->char* conversion to normalize the path
+  nsFilePath filePath(nativepath);
+  
+  PRUint32 cnt;
+  rv = serverArray->Count(&cnt);
+  NS_ENSURE_SUCCESS(rv, rv);
+  PRInt32 count = cnt;
+  PRInt32 i;
+  for (i=0; i<count; i++) 
+  {
+    
+    nsCOMPtr<nsIMsgIncomingServer> server = do_QueryElementAt(serverArray, i);
+    
+    if (!server) continue;
+    
+    // get the path string, convert it to an nsFilePath
+    nsCOMPtr<nsIFileSpec> nativeServerPath;
+    rv = server->GetLocalPath(getter_AddRefs(nativeServerPath));
+    if (NS_FAILED(rv)) continue;
+    
+    nsFileSpec spec;
+    nativeServerPath->GetFileSpec(&spec);
+    nsFilePath serverPath(spec);
+    
+    // check if filepath begins with serverPath
+    PRInt32 len = PL_strlen(serverPath);
+    if (PL_strncasecmp(serverPath, filePath, len) == 0) 
+    {
+      nsXPIDLCString serverURI;
+      rv = server->GetServerURI(getter_Copies(serverURI));
+      if (NS_FAILED(rv)) continue;
+      
+      // the relpath is just past the serverpath
+      const char *relpath = nativepath + len;
+      // skip past leading / if any
+      while (*relpath == '/')
+        relpath++;
+      
+      nsCAutoString pathStr(relpath);
+      PRInt32 sbdIndex;
+      while((sbdIndex = pathStr.Find(".sbd", PR_TRUE)) != -1)
+        pathStr.Cut(sbdIndex, 4);
+      
+      mailboxUri = serverURI;
+      mailboxUri.Append('/');
+      mailboxUri.Append(pathStr);
+      break;
+    }
+  }
+  return mailboxUri.IsEmpty() ? NS_ERROR_FAILURE : NS_OK;
+}
+
+NS_MSG_BASE void MsgStripQuotedPrintable (unsigned char *src)
+{
+  // decode quoted printable text in place
+  
+  if (!*src)
+    return;
+  unsigned char *dest = src;
+  int srcIdx = 0, destIdx = 0;
+  
+  while (src[srcIdx] != 0)
+  {
+    if (src[srcIdx] == '=')
+    {
+      unsigned char *token = &src[srcIdx];
+      unsigned char c = 0;
+      
+      // decode the first quoted char
+      if (token[1] >= '0' && token[1] <= '9')
+        c = token[1] - '0';
+      else if (token[1] >= 'A' && token[1] <= 'F')
+        c = token[1] - ('A' - 10);
+      else if (token[1] >= 'a' && token[1] <= 'f')
+        c = token[1] - ('a' - 10);
+      else
+      {
+        // first char after '=' isn't hex. check if it's a normal char
+        // or a soft line break.
+        if (src[srcIdx + 1] == nsCRT::CR || src[srcIdx + 1] == nsCRT::LF)
+          srcIdx++; // soft line break, ignore the '=';
+        else // normal char, copy it.
+          dest[destIdx++] = src[srcIdx++]; // aka token[0]
+        continue;
+      }
+      
+      // decode the second quoted char
+      c = (c << 4);
+      if (token[2] >= '0' && token[2] <= '9')
+        c += token[2] - '0';
+      else if (token[2] >= 'A' && token[2] <= 'F')
+        c += token[2] - ('A' - 10);
+      else if (token[2] >= 'a' && token[2] <= 'f')
+        c += token[2] - ('a' - 10);
+      else
+      {
+        // second char after '=' isn't hex. copy the '=' as a normal char and keep going
+        dest[destIdx++] = src[srcIdx++]; // aka token[0]
+        continue;
+      }
+      
+      // if we got here, we successfully decoded a quoted printable sequence,
+      // so bump each pointer past it and move on to the next char;
+      dest[destIdx++] = c; 
+      srcIdx += 3;
+      
+    }
+    else
+      dest[destIdx++] = src[srcIdx++];
+  }
+  
+  dest[destIdx] = src[srcIdx]; // null terminate
+}  
+
+
