@@ -69,6 +69,7 @@
 #include "nsSSLThread.h"
 #include "nsNSSShutDown.h"
 #include "nsNSSCertHelper.h"
+#include "nsNSSCleaner.h"
 
 #include "ssl.h"
 #include "secerr.h"
@@ -77,6 +78,7 @@
 #include "secasn1.h"
 #include "certdb.h"
 #include "cert.h"
+#include "keyhi.h"
 
 
 //#define DEBUG_SSL_VERBOSE //Enable this define to get minimal 
@@ -89,6 +91,8 @@
                        //we always write out to our own
                        //file.
 
+NSSCleanupAutoPtrClass(CERTCertificate, CERT_DestroyCertificate)
+
 /* SSM_UserCertChoice: enum for cert choice info */
 typedef enum {ASK, AUTO} SSM_UserCertChoice;
 
@@ -1322,16 +1326,14 @@ static PRInt32 PR_CALLBACK PSMRecv(PRFileDesc *fd, void *buf, PRInt32 amount,
     return -1;
   }
 
-  if (flags != PR_MSG_PEEK)
-  {
-    PR_SetError(PR_UNKNOWN_ERROR, 0);
-    return -1;
-  }
-
   nsNSSSocketInfo *socketInfo = (nsNSSSocketInfo*)fd->secret;
   NS_ASSERTION(socketInfo,"nsNSSSocketInfo was null for an fd");
 
-  return nsSSLThread::requestRecvMsgPeek(socketInfo, buf, amount, flags, timeout);
+  if (flags == PR_MSG_PEEK) {
+    return nsSSLThread::requestRecvMsgPeek(socketInfo, buf, amount, flags, timeout);
+  }
+
+  return fd->lower->methods->recv(fd->lower, buf, amount, flags, timeout);
 }
 
 static PRInt32 PR_CALLBACK PSMSend(PRFileDesc *fd, const void *buf, PRInt32 amount,
@@ -1343,8 +1345,7 @@ static PRInt32 PR_CALLBACK PSMSend(PRFileDesc *fd, const void *buf, PRInt32 amou
     return -1;
   }
 
-  PR_SetError(PR_UNKNOWN_ERROR, 0);
-  return -1;
+  return fd->lower->methods->send(fd->lower, buf, amount, flags, timeout);
 }
 
 static PRStatus PR_CALLBACK PSMConnectcontinue(PRFileDesc *fd, PRInt16 out_flags)
@@ -2096,6 +2097,26 @@ loser:
 	return ret;
 }
 
+static PRBool hasExplicitKeyUsageNonRepudiation(CERTCertificate *cert)
+{
+  /* There is no extension, v1 or v2 certificate */
+  if (!cert->extensions)
+    return PR_FALSE;
+
+  SECStatus srv;
+  SECItem keyUsageItem;
+  keyUsageItem.data = NULL;
+
+  srv = CERT_FindKeyUsageExtension(cert, &keyUsageItem);
+  if (srv == SECFailure)
+    return PR_FALSE;
+
+  unsigned char keyUsage = keyUsageItem.data[0];
+  PORT_Free (keyUsageItem.data);
+
+  return (keyUsage & KU_NON_REPUDIATION);
+}
+
 /*
  * Function: SECStatus SSM_SSLGetClientAuthData()
  * Purpose: this callback function is used to pull client certificate
@@ -2179,7 +2200,7 @@ SECStatus nsNSS_SSLGetClientAuthData(void* arg, PRFileDesc* socket,
 
     /* find all user certs that are valid and for SSL */
     certList = CERT_FindUserCertsByUsage(CERT_GetDefaultCertDB(), 
-                                         certUsageSSLClient, PR_TRUE,
+                                         certUsageSSLClient, PR_FALSE,
                                          PR_TRUE, wincx);
     if (certList == NULL) {
       goto noCert;
@@ -2198,6 +2219,9 @@ SECStatus nsNSS_SSLGetClientAuthData(void* arg, PRFileDesc* socket,
       goto noCert;
     }
 
+    CERTCertificate* low_prio_nonrep_cert = NULL;
+    CERTCertificateCleaner low_prio_cleaner(low_prio_nonrep_cert);
+
     /* loop through the list until we find a cert with a key */
     while (!CERT_LIST_END(node, certList)) {
       /* if the certificate has restriction and we do not satisfy it
@@ -2213,9 +2237,18 @@ SECStatus nsNSS_SSLGetClientAuthData(void* arg, PRFileDesc* socket,
 
       privKey = PK11_FindKeyByAnyCert(node->cert, wincx);
       if (privKey != NULL) {
-          /* this is a good cert to present */
+        if (hasExplicitKeyUsageNonRepudiation(node->cert)) {
+          SECKEY_DestroyPrivateKey(privKey);
+          privKey = NULL;
+          // Not a prefered cert
+          if (!low_prio_nonrep_cert) // did not yet find a low prio cert
+            low_prio_nonrep_cert = CERT_DupCertificate(node->cert);
+        }
+        else {
+          // this is a good cert to present
           cert = CERT_DupCertificate(node->cert);
           break;
+        }
       }
       keyError = PR_GetError();
       if (keyError == SEC_ERROR_BAD_PASSWORD) {
@@ -2226,6 +2259,12 @@ SECStatus nsNSS_SSLGetClientAuthData(void* arg, PRFileDesc* socket,
       node = CERT_LIST_NEXT(node);
     }
 
+    if (!cert && low_prio_nonrep_cert) {
+      cert = low_prio_nonrep_cert;
+      low_prio_nonrep_cert = NULL; // take it away from the cleaner
+      privKey = PK11_FindKeyByAnyCert(cert, wincx);
+    }
+
     if (cert == NULL) {
         goto noCert;
     }
@@ -2341,6 +2380,9 @@ SECStatus nsNSS_SSLGetClientAuthData(void* arg, PRFileDesc* socket,
       if (NS_FAILED(tempCert->FormatUIStrings(i_nickname, nickWithSerial, details)))
         continue;
 
+      if (hasExplicitKeyUsageNonRepudiation(node->cert))
+        nickWithSerial.Append(NS_LITERAL_STRING(" [NR]"));
+
       certNicknameList[CertsToUse] = ToNewUnicode(nickWithSerial);
       if (!certNicknameList[CertsToUse])
         continue;
